-- 表结构创建SQL
-- 创建数据库
CREATE DATABASE IF NOT EXISTS library_db
CHARACTER SET utf8mb4
COLLATE utf8mb4_unicode_ci;

USE library_db;

-- 用户表
CREATE TABLE users (
    user_id INT AUTO_INCREMENT PRIMARY KEY,
    username VARCHAR(50) UNIQUE NOT NULL,
    password VARCHAR(255) NOT NULL,
    real_name VARCHAR(50) NOT NULL,
    email VARCHAR(100) UNIQUE,
    phone VARCHAR(20),
    role ENUM('admin', 'librarian', 'user') DEFAULT 'user',
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
);

-- 出版社表
CREATE TABLE publishers (
    publisher_id INT AUTO_INCREMENT PRIMARY KEY,
    name VARCHAR(100) UNIQUE NOT NULL,
    address VARCHAR(200),
    contact_phone VARCHAR(20),
    contact_email VARCHAR(100)
);

-- 分类表
CREATE TABLE categories (
    category_id INT AUTO_INCREMENT PRIMARY KEY,
    name VARCHAR(50) UNIQUE NOT NULL,
    parent_id INT DEFAULT NULL,
    FOREIGN KEY (parent_id) REFERENCES categories(category_id) ON DELETE SET NULL
);

-- 图书表
CREATE TABLE books (
    book_id INT AUTO_INCREMENT PRIMARY KEY,
    isbn VARCHAR(13) UNIQUE NOT NULL,
    title VARCHAR(200) NOT NULL,
    author VARCHAR(100) NOT NULL,
    publisher_id INT NOT NULL,
    publish_date DATE,
    price DECIMAL(10,2),
    total_copies INT DEFAULT 1 CHECK (total_copies >= 0),
    available_copies INT DEFAULT 1 CHECK (available_copies >= 0),
    category_id INT NOT NULL,
    location VARCHAR(50),
    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    FOREIGN KEY (publisher_id) REFERENCES publishers(publisher_id),
    FOREIGN KEY (category_id) REFERENCES categories(category_id)
);

-- 借阅记录表
CREATE TABLE borrow_records (
    record_id INT AUTO_INCREMENT PRIMARY KEY,
    user_id INT NOT NULL,
    book_id INT NOT NULL,
    borrow_date DATE NOT NULL,
    due_date DATE NOT NULL,
    return_date DATE,
    status ENUM('borrowed', 'returned', 'overdue') DEFAULT 'borrowed',
    FOREIGN KEY (user_id) REFERENCES users(user_id),
    FOREIGN KEY (book_id) REFERENCES books(book_id),
    INDEX idx_borrow_user (user_id),
    INDEX idx_borrow_book (book_id)
);

-- 预约表
CREATE TABLE reservations (
    reservation_id INT AUTO_INCREMENT PRIMARY KEY,
    user_id INT NOT NULL,
    book_id INT NOT NULL,
    reserve_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    status ENUM('pending', 'fulfilled', 'canceled') DEFAULT 'pending',
    FOREIGN KEY (user_id) REFERENCES users(user_id),
    FOREIGN KEY (book_id) REFERENCES books(book_id),
    UNIQUE KEY unique_reservation (user_id, book_id),
    INDEX idx_reserve_user (user_id),
    INDEX idx_reserve_book (book_id)
);

-- 触发器：更新图书可借数量
DELIMITER $$
CREATE TRIGGER after_borrow
AFTER INSERT ON borrow_records
FOR EACH ROW
BEGIN
    UPDATE books
    SET available_copies = available_copies - 1
    WHERE book_id = NEW.book_id;
END$$

CREATE TRIGGER after_return
AFTER UPDATE ON borrow_records
FOR EACH ROW
BEGIN
    IF NEW.return_date IS NOT NULL AND OLD.return_date IS NULL THEN
        UPDATE books
        SET available_copies = available_copies + 1
        WHERE book_id = NEW.book_id;
    END IF;
END$$
DELIMITER ;

-- 索引优化
-- 添加复合索引
CREATE INDEX idx_book_search ON books(title, author);
CREATE INDEX idx_user_activity ON borrow_records(user_id, borrow_date);

-- 视图
-- 当前借阅视图
CREATE VIEW current_borrows AS
SELECT u.username, b.title, br.borrow_date, br.due_date
FROM borrow_records br
JOIN users u ON br.user_id = u.user_id
JOIN books b ON br.book_id = b.book_id
WHERE br.status = 'borrowed';

-- 图书库存视图
CREATE VIEW book_inventory AS
SELECT
    b.title,
    b.total_copies,
    b.available_copies,
    (b.total_copies - b.available_copies) AS borrowed_count
FROM books b;

-- 存储过程
DELIMITER $$
CREATE PROCEDURE borrow_book(
    IN p_user_id INT,
    IN p_book_id INT,
    IN p_borrow_days INT
)
BEGIN
    DECLARE available INT;

    START TRANSACTION;

    SELECT available_copies INTO available
    FROM books WHERE book_id = p_book_id FOR UPDATE;

    IF available > 0 THEN
        INSERT INTO borrow_records
        (user_id, book_id, borrow_date, due_date)
        VALUES (
            p_user_id,
            p_book_id,
            CURDATE(),
            CURDATE() + INTERVAL p_borrow_days DAY
        );

        UPDATE books
        SET available_copies = available_copies - 1
        WHERE book_id = p_book_id;

        COMMIT;
    ELSE
        ROLLBACK;
        SIGNAL SQLSTATE '45000'
            SET MESSAGE_TEXT = 'No available copies';
    END IF;
END$$
DELIMITER ;