import pygame
import random
import sys

# 初始化Pygame
pygame.init()

# 游戏常量
WIDTH = 800
HEIGHT = 600
GRID_SIZE = 20
FPS = 10

# 颜色定义
BLACK = (0, 0, 0)
WHITE = (255, 255, 255)
RED = (255, 0, 0)
GREEN = (0, 255, 0)
BLUE = (0, 0, 255)

# 初始化游戏窗口
screen = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("贪吃蛇")
clock = pygame.time.Clock()


class Snake:
    def __init__(self):
        self.reset()

    def reset(self):
        self.body = [(WIDTH // 2, HEIGHT // 2)]
        self.direction = (0, 0)  # 初始静止
        self.grow = False
        self.score = 0

    def move(self):
        if self.direction == (0, 0):
            return  # 游戏未开始

        head_x, head_y = self.body[0]
        new_head = (
            head_x + self.direction[0] * GRID_SIZE,
            head_y + self.direction[1] * GRID_SIZE
        )

        self.body.insert(0, new_head)
        if not self.grow:
            self.body.pop()
        else:
            self.grow = False
            self.score += 10

    def check_collision(self):
        # 边界检测
        head = self.body[0]
        if (head[0] < 0 or head[0] >= WIDTH or
                head[1] < 0 or head[1] >= HEIGHT):
            return True

        # 自碰撞检测
        return head in self.body[1:]

    def draw(self):
        for i, segment in enumerate(self.body):
            color = GREEN if i == 0 else BLUE
            pygame.draw.rect(
                screen,
                color,
                (segment[0], segment[1], GRID_SIZE - 1, GRID_SIZE - 1)
            )


class Food:
    def __init__(self, snake):
        self.position = (0, 0)
        self.spawn(snake)

    def spawn(self, snake):
        while True:
            x = random.randrange(0, WIDTH, GRID_SIZE)
            y = random.randrange(0, HEIGHT, GRID_SIZE)
            if (x, y) not in snake.body:
                self.position = (x, y)
                break

    def draw(self):
        pygame.draw.rect(
            screen,
            RED,
            (self.position[0], self.position[1], GRID_SIZE - 1, GRID_SIZE - 1)
        )


def draw_score(score):
    font = pygame.font.Font(None, 36)
    text = font.render(f"Score: {score}", True, WHITE)
    screen.blit(text, (10, 10))


def game_over():
    font = pygame.font.Font(None, 72)
    text = font.render("Game Over!", True, RED)
    text_rect = text.get_rect(center=(WIDTH // 2, HEIGHT // 2))

    font_small = pygame.font.Font(None, 36)
    instruction = font_small.render("Press R to restart, Q to quit", True, WHITE)
    inst_rect = instruction.get_rect(center=(WIDTH // 2, HEIGHT // 2 + 50))

    screen.blit(text, text_rect)
    screen.blit(instruction, inst_rect)
    pygame.display.flip()

    while True:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_r:
                    return True
                if event.key == pygame.K_q:
                    pygame.quit()
                    sys.exit()


def main():
    snake = Snake()
    food = Food(snake)
    running = True

    while running:
        # 事件处理
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False

            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_UP and snake.direction != (0, 1):
                    snake.direction = (0, -1)
                elif event.key == pygame.K_DOWN and snake.direction != (0, -1):
                    snake.direction = (0, 1)
                elif event.key == pygame.K_LEFT and snake.direction != (1, 0):
                    snake.direction = (-1, 0)
                elif event.key == pygame.K_RIGHT and snake.direction != (-1, 0):
                    snake.direction = (1, 0)

        # 游戏逻辑
        snake.move()

        # 食物检测
        if snake.body[0] == food.position:
            snake.grow = True
            food.spawn(snake)

        # 碰撞检测
        if snake.check_collision():
            if game_over():
                snake.reset()
                food.spawn(snake)
            else:
                running = False

        # 绘制画面
        screen.fill(BLACK)
        snake.draw()
        food.draw()
        draw_score(snake.score)
        pygame.display.flip()
        clock.tick(FPS)

    pygame.quit()
    sys.exit()


if __name__ == "__main__":
    main()